//
//  ArticleViewController.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 18.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//

#import "ArticleViewController.h"
#import "HorusUtils.h"
#import "Order+CoreDataClass.h"
#import "ModelStack.h"

@interface ArticleViewController ()

@end

@implementation ArticleViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.descriptionTextView.textContainer.lineFragmentPadding = 0;
    self.commentsOrderTextView.textContainer.lineFragmentPadding = 0;
    self.descriptionTextView.textContainerInset = UIEdgeInsetsZero;
    self.commentsOrderTextView.textContainerInset = UIEdgeInsetsZero;
    self.commentsOrderTextView.delegate = self;
    [self.commentsOrderTextView endEditing:YES];
    
    self.addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd
                                                                   target:self action:@selector(addArticleToOrder:)];
    
    [self syncViewToModel];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
 #pragma mark - Navigation
 
 // In a storyboard-based application, you will often want to do a little preparation before navigation
 - (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
 // Get the new view controller using [segue destinationViewController].
 // Pass the selected object to the new view controller.
 }
 */

-(void)syncViewToModel
{
    if(self.orderLineModel != nil)  //This means it's an article which is already in the cart
    {
        self.model = (Article*)self.orderLineModel.article;
        self.margin = [NSNumber numberWithInteger: self.orderLineModel.margin];
        self.amountStepper.value = self.orderLineModel.amount;
        
        if ((self.orderLineModel.comments == nil ||
             [self.orderLineModel.comments  isEqual: @""]) &&
            self.orderLineModel.order == [Order activeOrder]) {
            self.commentsOrderTextView.text = DEFAULT_COMMENTS;
        }
        else{
            self.commentsOrderTextView.text = self.orderLineModel.comments;
        }
        
        if(self.orderLineModel.order != [Order activeOrder])
        {
            self.amountStepper.hidden = true;
            self.commentsOrderTextView.userInteractionEnabled = false;
        }
    }
    else //Article from the catalog
    {
        [self.navigationItem setRightBarButtonItems:@[self.addButton] animated:YES];
        self.commentsOrderTextView.text = DEFAULT_COMMENTS;
        self.amountStepper.value = 1;
    }
    
    self.title = self.model.articleNumber;
    
    NSNumber* endPrice = [NSNumber numberWithFloat:self.model.price * (1 + 0.01 * [self.margin floatValue])];
    
    self.priceLabel.text = [HorusUtils priceString:endPrice units:YES];
    self.descriptionTextView.text = self.model.descriptionText;
    self.pictureImage.image = self.model.picture;
    
    self.amountLabel.text = [NSString stringWithFormat:@"%.0f", self.amountStepper.value];
}

-(IBAction) stepperPressed:(id)sender
{
    UIStepper *stepper = (UIStepper *) sender;
    
    self.amountLabel.text = [NSString stringWithFormat:@"%.0f", stepper.value];
    
    if(self.orderLineModel != nil)
    {
        self.orderLineModel.amount = stepper.value;
        [[ModelStack sharedInstance] saveContext];
    }
}

-(IBAction) handleImageSwipe:(UISwipeGestureRecognizer *) sender
{
    [self.navigationController popViewControllerAnimated:YES];
}

-(IBAction) addArticleToOrder:(id)sender
{
    [[Order activeOrder] addArticleToOrder:self.model
                                    amount:[NSNumber numberWithInt:(int)[self.amountLabel.text integerValue]]
                                  comments:[self.commentsOrderTextView.text isEqual: DEFAULT_COMMENTS]?nil:self.commentsOrderTextView.text
                                    margin:self.margin];
    
    [self.navigationController popViewControllerAnimated:YES];
}

#pragma mark - UITextViewDelegate

-(void)textViewDidBeginEditing:(UITextView *)textView
{
    [UIView animateWithDuration:.5f animations:^{
        [self.view setFrame:CGRectMake(self.view.frame.origin.x,self.view.frame.origin.y-200,self.view.frame.size.width,self.view.frame.size.height)];
    }];
    
    if ([textView.text  isEqual: DEFAULT_COMMENTS]) {
        textView.text = @"";
    }
}

-(void)textViewDidEndEditing:(UITextView *)textView
{
    [UIView animateWithDuration:.5f animations:^{
        [self.view setFrame:CGRectMake(self.view.frame.origin.x,self.view.frame.origin.y+200,self.view.frame.size.width,self.view.frame.size.height)];
    }];
    
    if ([textView.text  isEqual: @""]) {
        textView.text = DEFAULT_COMMENTS;
    }
    
    if(self.orderLineModel != nil)
    {
        self.orderLineModel.comments = [self.commentsOrderTextView.text isEqual: DEFAULT_COMMENTS]?@"":self.commentsOrderTextView.text;
    }
}
- (IBAction)hideKeyboard:(id)sender {
    [self.commentsOrderTextView endEditing:YES];
}

@end
