//
//  CatalogViewController.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 04.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//

#import "CatalogViewController.h"
#import "CatalogTableViewController.h"
#import "CatalogCollectionViewController.h"
#import "Catalog+CoreDataClass.h"
#import "SSPieProgressView.h"
#import "ArticleTableViewCell.h"
#import "ArticleCollectionViewCell.h"
#import "HorusUtils.h"
#import "ArticleCatalog+CoreDataClass.h"
#import "ArticleViewController.h"
#import "OrderViewController.h"

@interface CatalogViewController ()

@property (nonatomic,strong) NSTimer *searchDelayer;
@property (nonatomic, retain) NSMutableArray * searchResults;

@end

@implementation CatalogViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    [Catalog defaultCatalogModel].delegate = self;
    
    self.rightButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh
                                                                     target:self
                                                                     action:@selector(refreshCatalogFromServer)];
    
    [self.navigationItem setLeftBarButtonItem:self.rightButton animated:YES];
    
    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
    {
        self.searchBar = [[UISearchBar alloc] initWithFrame:CGRectMake(0.0, 0.0, self.view.bounds.size.width / 4, 44.0)];
        
        UIBarButtonItem *navRight = [[UIBarButtonItem alloc] initWithCustomView:self.searchBar];
        [[self navigationItem] setRightBarButtonItem:navRight];
        
        self.mainCatalogViewController = [[CatalogCollectionViewController alloc] init];
        self.mainCatalogViewController.view.frame = [[UIScreen mainScreen] bounds];
    }
    else
    {
        self.searchBar = [[UISearchBar alloc] initWithFrame:CGRectMake(0.0, 0.0, self.view.bounds.size.width, 44.0)];
        self.searchBar.showsCancelButton=YES;
        
        self.searchButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSearch
                                                                          target:self
                                                                          action:@selector(showSearchBar)];
        
        [self.navigationItem setRightBarButtonItem:self.searchButton animated:YES];
        self.mainCatalogViewController = [[CatalogTableViewController alloc] initWithNibName:nil bundle:nil];
        self.mainCatalogViewController.view.frame = [[UIScreen mainScreen] bounds];
    }
    
    self.searchBar.delegate = self;
    self.searchBar.showsCancelButton = YES;
    self.searchBar.keyboardType = UIKeyboardTypeNumberPad;
    
    [self setViewControllersDelegates];
    
    [self addChildViewController:self.mainCatalogViewController];
    [self.view addSubview:self.mainCatalogViewController.view];
    
    [self handleSearch];
    
    //This allows us to update the margin when the user changes it in Settings
    [[NSNotificationCenter defaultCenter]addObserver:self
                                            selector:@selector(updateMargin)
                                                name:UIApplicationDidBecomeActiveNotification
                                              object:nil];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void)viewWillAppear:(BOOL)animated
{
    
    
}

-(void) setViewControllersDelegates
{
    if([self.mainCatalogViewController isKindOfClass:[CatalogCollectionViewController class]])
    {
        CatalogCollectionViewController *catalogVC = (CatalogCollectionViewController*)self.mainCatalogViewController;
        catalogVC.collectionView.delegate = self;
        catalogVC.collectionView.dataSource = self;
    }
    else if([self.mainCatalogViewController isKindOfClass:[CatalogTableViewController class]])
    {
        CatalogTableViewController *catalogVC = (CatalogTableViewController*)self.mainCatalogViewController;
        catalogVC.tableView.delegate = self;
        catalogVC.tableView.dataSource = self;
    }
}

#pragma mark - Refresh data

-(void) updateTitleMargin
{
    self.title = [NSString stringWithFormat:@"Catálogo (%@)",self.margin];
    self.tabBarItem.title = @"Catálogo";
}

-(void) updateMargin
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults synchronize];
    
    NSNumberFormatter * f = [[NSNumberFormatter alloc] init];
    [f setNumberStyle:NSNumberFormatterDecimalStyle];
    [self setMargin: [f numberFromString:[[NSUserDefaults standardUserDefaults] objectForKey:@"margin"]]];
    
    if (self.margin == nil || [self.margin floatValue]<0)  //In case the user introduces a wrong margin value in Settings
    {
        UIAlertController * alert = [UIAlertController
                                     alertControllerWithTitle:@"Tarifa errónea"
                                     message:@"La tarifa introducida en los ajustes no es válida"
                                     preferredStyle:UIAlertControllerStyleAlert];
        
        [alert addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
            textField.placeholder = @"";
            textField.text = [[NSUserDefaults standardUserDefaults] objectForKey:@"margin"];
            textField.keyboardType= UIKeyboardTypeNumberPad;
        }];
        
        UIAlertAction *confirmAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            
            NSNumberFormatter * f = [[NSNumberFormatter alloc] init];
            [f setNumberStyle:NSNumberFormatterDecimalStyle];
            NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
            [defaults setObject:[NSString stringWithFormat:@"%@",[f numberFromString:[[alert textFields][0] text]]]
                         forKey:@"margin"];
            [defaults synchronize];
            [self updateMargin];
            
        }];
        [alert addAction:confirmAction];
        
        [self presentViewController:alert animated:YES completion:nil];
        
    }else{
        NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
        [defaults setObject:[NSString stringWithFormat:@"%@",self.margin]
                     forKey:@"margin"];
        [defaults synchronize];
        //[self updateTitleMargin];  //Shows margin in title bar without '%', discuss later
        [self reloadData];
    }
}

-(void) refreshCatalogFromServer
{
    [[Catalog defaultCatalogModel] refreshFromServer];
}

-(void) reloadData
{
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
    
    if([self.mainCatalogViewController isKindOfClass:[CatalogTableViewController class]])
    {
        CatalogTableViewController *catalogVC = (CatalogTableViewController*)self.mainCatalogViewController;
        [catalogVC.tableView reloadData];
        if(self.searchResults.count > 0)
        {
            [catalogVC.tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionTop animated:YES];
        }
    }else if([self.mainCatalogViewController isKindOfClass:[CatalogCollectionViewController class]])
    {
        CatalogCollectionViewController *catalogVC = (CatalogCollectionViewController*) self.mainCatalogViewController;
        [catalogVC.collectionView reloadData];
        [catalogVC.collectionView scrollRectToVisible:CGRectMake(0, 0, 1, 1) animated:YES];
        if(self.searchResults.count > 0)
        {
            [catalogVC.collectionView scrollToItemAtIndexPath:indexPath atScrollPosition:UICollectionViewScrollPositionTop animated:YES];
        }
    }
}

#pragma mark - Search Bar delegate methods

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    [self performSelectorInBackground:@selector(handleSearch) withObject:nil];
    [searchBar resignFirstResponder];
}

-(void) showSearchBar{
    [self.navigationItem setRightBarButtonItem:nil animated:YES];
    [self.navigationItem setLeftBarButtonItem:nil animated:YES];
    
    self.navigationItem.titleView = self.searchBar;
    [self.searchBar becomeFirstResponder];
}

- (void)searchBar:(UISearchBar *)searchBar
    textDidChange:(NSString *)searchText
{
    [self.searchDelayer invalidate];
    self.searchDelayer = [NSTimer scheduledTimerWithTimeInterval:0.5
                                                          target:self
                                                        selector:@selector(handleSearch)
                                                        userInfo:nil
                                                         repeats:NO];
    
    [searchBar becomeFirstResponder];
    
}

- (void)handleSearch
{
    NSString* text = self.searchBar.text;
    
    //Search is async in for performance reasons
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        NSSortDescriptor *sort = [NSSortDescriptor sortDescriptorWithKey:@"articleNumber" ascending:YES];
        
        if([text length] == 0)
        {
            self.searchResults = [[[[[Catalog defaultCatalogModel] articleCatalog] allObjects]sortedArrayUsingDescriptors:@[sort]]mutableCopy];
            
        }else
        {
            NSPredicate *predicate = [NSPredicate predicateWithFormat:@"articleNumber beginswith[c] %@",text];
            self.searchResults = [[[[[Catalog defaultCatalogModel].articleCatalog filteredSetUsingPredicate:predicate] allObjects]sortedArrayUsingDescriptors:@[sort]]mutableCopy];
        }
        
        dispatch_async(dispatch_get_main_queue(), ^{
            [self reloadData];
        });
    });
}

- (void)searchBarCancelButtonClicked:(UISearchBar *) searchBar
{
    [searchBar resignFirstResponder];
    self.navigationItem.titleView = nil;
    [self.navigationItem setRightBarButtonItem:self.searchButton animated:YES];
    self.searchBar.text = @"";
    [self.navigationItem setLeftBarButtonItem:self.rightButton animated:YES];
    [self handleSearch];
}

- (void) scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    [self.searchBar resignFirstResponder];
}


#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if ([segue.identifier isEqualToString:@"articleCatalogSegue"] &&
        self.indexPath) {
        ArticleViewController *destViewController = segue.destinationViewController;
        destViewController.margin = self.margin;
        destViewController.model = [self.searchResults objectAtIndex:self.indexPath.row];
    }
}

#pragma mark - CatalogDelegate

- (void)didChangeProgress:(NSNumber * _Nonnull)aProgress
{
    SSPieProgressView *pie = (SSPieProgressView*)self.navigationItem.leftBarButtonItem.customView;
    [pie setProgress:(CGFloat)[aProgress floatValue]];
}

- (void)didErrorDownloadingCatalog:(NSString * _Nonnull)anError
{
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
    
    self.rightButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh
                                                                     target:self
                                                                     action:@selector(refreshCatalogFromServer)];
    
    [self.navigationItem setLeftBarButtonItem:self.rightButton animated:YES];
    
    UIAlertController * alert = [UIAlertController
                                 alertControllerWithTitle:@"Error actualizando catalogo"
                                 message:anError
                                 preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction* yesButton = [UIAlertAction
                                actionWithTitle:@"Entendido"
                                style:UIAlertActionStyleDefault
                                handler:nil];
    [alert addAction:yesButton];
    
    [self presentViewController:alert animated:YES completion:nil];
    
    [self.navigationItem.leftBarButtonItem setEnabled:YES];
}

- (void)didFinishDownloadingCatalog
{
    //This is very important in order to not let the user add an Article to the catalog that it's not in the db any longer!
    [[self navigationController] popToRootViewControllerAnimated:YES];
    self.rightButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh
                                                                     target:self
                                                                     action:@selector(refreshCatalogFromServer)];
    
    [self.navigationItem setLeftBarButtonItem:self.rightButton animated:YES];
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:NO];
    [[UIApplication sharedApplication] setIdleTimerDisabled: NO];
    [self.navigationItem.leftBarButtonItem setEnabled:YES];
    [self handleSearch];
}

- (void)didStartDownloadingCatalog {
    SSPieProgressView *pie = [[SSPieProgressView alloc] initWithFrame:(CGRectMake(0, 0, 25, 25))];
    [pie setPieBorderColor:self.navigationController.navigationBar.tintColor];
    [pie setPieFillColor:self.navigationController.navigationBar.tintColor];
    [pie setPieBackgroundColor:[UIColor clearColor]];
    UIBarButtonItem *pieUIBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView:pie];
    
    self.rightButton = pieUIBarButtonItem;
    
    [[UIApplication sharedApplication] setIdleTimerDisabled: YES];
    
    [self.navigationItem setLeftBarButtonItem:self.rightButton animated:YES];
}

- (void)willStartDownloadingCatalog
{
    [[UIApplication sharedApplication] setNetworkActivityIndicatorVisible:YES];
    [self.navigationItem.leftBarButtonItem setEnabled:NO];
}

#pragma mark - Collection and Table View delegate common methods

-(NSInteger) numberOfSections
{
    return 1;
}

-(NSInteger) numberOfItems
{
    return self.searchResults.count;
}

-(void) didSelectItemAtIndexPath:(NSIndexPath *) indexPath
{
    self.indexPath = indexPath;  //We save the selected indexPath to use it in prepareForSegue
    [self performSegueWithIdentifier:@"articleCatalogSegue" sender:self];
}

#pragma mark - Collection View delegate

-(NSInteger) numberOfSectionsInCollectionView:(UICollectionView *)collectionView
{
    return [self numberOfSections];
}

-(NSInteger) collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    return [self numberOfItems];
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath
{
    [self didSelectItemAtIndexPath:indexPath];
}

-(UICollectionViewCell *) collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{    
    ArticleCollectionViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:@"CustomCellReuseID"
                                                                                forIndexPath:indexPath];
    
    ArticleCatalog* selectedArticle = [self.searchResults objectAtIndex:indexPath.row];
    [cell setData:selectedArticle withMargin:self.margin];
    
    return cell;
}

- (CGSize)collectionView:(UICollectionView *)collectionView
                  layout:(UICollectionViewLayout*)collectionViewLayout
  sizeForItemAtIndexPath:(NSIndexPath *)indexPath
{
    UIView* cellView = [[[UINib nibWithNibName:@"ArticleCollectionViewCell"
                                        bundle:[NSBundle mainBundle]] instantiateWithOwner:nil
                         options:nil] lastObject];
    
    return cellView.frame.size;
}

#pragma mark - Table view data source

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [self didSelectItemAtIndexPath:indexPath];
}

- (nonnull UITableViewCell *)tableView:(nonnull UITableView *)tableView cellForRowAtIndexPath:(nonnull NSIndexPath *)indexPath
{
    ArticleTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"articleTableCell"];
    if (cell == nil) {
        cell = [[ArticleTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault
                                           reuseIdentifier:@"CustomCellReuseID"];
    }
    
    ArticleCatalog* selectedArticle = [self.searchResults objectAtIndex:indexPath.row];
    [cell setData:selectedArticle withMargin:self.margin];
    
    return cell;
}

- (NSInteger)tableView:(nonnull UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self numberOfItems];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 75.0;
}

@end
