//
//  OrderTableViewCell.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 25.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//

#import "OrderTableViewCell.h"
#import "Order+CoreDataClass.h"
#import "OrderLine+CoreDataClass.h"
#import "ArticleOrder+CoreDataClass.h"
#import "HorusUtils.h"

@implementation OrderTableViewCell

- (void)awakeFromNib {
    [super awakeFromNib];
    // Initialization code
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
    [super setSelected:selected animated:animated];
    // Configure the view for the selected state
}

-(void)setData:(Order *)order {
    
    self.model = order;
    
    self.orderIdLabel.text = [order guiOrderId];
    self.customerNameLabel.text = [HorusUtils contactStringWith: order.customerId details:ContactNameCompany];
    
    self.dateLabel.text = [NSDateFormatter localizedStringFromDate:order.date
                                                         dateStyle:NSDateFormatterMediumStyle
                                                         timeStyle:NSDateFormatterNoStyle];
    self.articlesTotalLabel.text = [NSString stringWithFormat:@"%lu %@",(long)[order total]?(long)[[order totalArticles]integerValue]:0,(long)[[order totalArticles] integerValue] == 1 ? @"pieza" : @"piezas"];
    
    self.priceTotalLabel.text = [HorusUtils priceString:order.total units:YES];
    self.imageView.image = [self imagesMosaic:order];
}

-(UIImage *)imagesMosaic:(Order *)order
{
    CGRect contentRect = self.contentView.bounds;
    contentRect.size.width = 75;
    contentRect.size.height = 75;
    
    NSArray * array= [order.line allObjects];
    
    OrderLine *line;
    
    if ([array count] == 0)
    {
        return nil;
    }
    else
    {
        UIGraphicsBeginImageContext(contentRect.size);
        NSInteger side = ceilf(sqrt([array count]));
        
        if ([array count] > 1) {
            contentRect.size.height = contentRect.size.height / side;
            contentRect.size.width = contentRect.size.width / side;
        }
        
        NSInteger i, index, j;
        for (i = 0, index = 0; i < side; i++)
        {
            for (j = 0; j < side && index < [array count]; j++)
            {
                line = [array objectAtIndex:index];
                
                contentRect.origin.x = j * contentRect.size.width;
                contentRect.origin.y = i * contentRect.size.height;
                
                [line.article.picture drawInRect:contentRect];
                index++;
            }
        }
        
        UIImage *destImage = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
        return destImage;
    }
}

- (void)layoutSubviews {
    
    [super layoutSubviews];
    
    // Slide image to the left, eliminating gutter
    self.imageView.frame = CGRectMake(0, 0, self.imageView.frame.size.width, self.imageView.frame.size.height);
}

@end
