//
//  OrderViewController.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 19.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//

#import "OrderViewController.h"
#import "HorusUtils.h"
#import "OrderLineTableViewCell.h"
#import "ModelStack.h"
#import "ArticleOrder+CoreDataClass.h"
#import "ArticleViewController.h"
#import "PdfViewController.h"

@interface OrderViewController ()

@property (nonatomic) CGPoint tempXYPan;
@property (nonatomic) UIBarButtonItem* saveButton;
@property (nonatomic) UIBarButtonItem* pdfButton;
@property (nonatomic) UIBarButtonItem* contactButton;
@property (nonatomic) NSIndexPath * indexPath;
@property (nonatomic) bool editable;

@end

@implementation OrderViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.pdfButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAction
                                                                   target:self action:@selector(viewPDF)];
    
    if(self.model == nil)  // This means we are showing the cart
    {
        self.saveButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone
                                                                        target:self action:@selector(askNewOrder)];
        
        self.contactButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemBookmarks
                                                                           target:self action:@selector(selectCustomer)];
        
        self.model = [Order activeOrder];
        
        ModelStack *sharedDataStack = [ModelStack sharedInstance];
        NSManagedObjectContext *context = sharedDataStack.persistentContainer.viewContext;
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updateData)
                                                     name:NSManagedObjectContextDidSaveNotification
                                                   object:context];
        
        self.title = @"Carrito";
        [self setEditable: true];
    }
    else // This means we are showing an old order
    {
        self.title = @"Pedido";
        [self setEditable: false];
    }
        
    self.orderCommentsTextView.delegate = self;
    
    self.contactDetailsLabel.textContainer.lineFragmentPadding = 0;
    self.contactDetailsLabel.textContainerInset = UIEdgeInsetsZero;
    
    self.orderCommentsTextView.textContainer.lineFragmentPadding = 0;
    self.orderCommentsTextView.textContainerInset = UIEdgeInsetsZero;
    
    self.articlesTable.delegate = self;
    self.articlesTable.dataSource = self;
    self.articlesTable.rowHeight = 70.0;
    [self.articlesTable registerNib:[UINib nibWithNibName:@"OrderLineTableViewCell" bundle:nil]
             forCellReuseIdentifier:@"articleOrderTableCell"];
}

- (void) viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self updateData];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)setEditable: (bool) editable
{
    _editable = editable;
    self.panGestureRecognizer.enabled = editable;
    self.discountUpButton.hidden = !editable;
    self.discountDownButton.hidden = !editable;
    self.orderCommentsTextView.userInteractionEnabled = editable;
    self.articlesTable.allowsMultipleSelectionDuringEditing = NO;
    
    self.articlesTable.editing = NO;
    
    if(editable)
    {
        [self.navigationItem setRightBarButtonItems:@[self.pdfButton,self.saveButton] animated:YES];
        [self.navigationItem setLeftBarButtonItems:@[self.contactButton] animated:YES];
    }
    else
    {
        [self.navigationItem setRightBarButtonItems:@[self.pdfButton] animated:YES];
    }
}

-(void) updateData
{
    [self.articlesTable reloadData];
    self.totalWithoutDiscount.text = [HorusUtils  priceString: self.model.totalWithoutDiscount units:YES];
    self.discountTotalLabel.text = [HorusUtils priceString: self.model.discountAmount units:YES];
    self.totalLabel.text = [HorusUtils priceString:self.model.total units:YES];
    self.discountLabel.text = [NSString stringWithFormat:@"%lu", (long)self.model.discount];
    self.piezasLabel.text = [NSString stringWithFormat:@"%lu %@",(long)[self.model total]?(long)[[self.model totalArticles]integerValue]:0,(long)[[self.model totalArticles] integerValue] == 1 ? @"pieza" : @"piezas"];
    self.dateLabel.text = [NSDateFormatter localizedStringFromDate:self.model.date
                                                         dateStyle:NSDateFormatterMediumStyle
                                                         timeStyle:NSDateFormatterNoStyle];
    self.orderIDLabel.text = [self.model guiOrderId];
    self.orderCommentsTextView.text = [self.model comments];
    
    self.contactDetailsLabel.text = [HorusUtils contactStringWith:self.model.customerId
                                                          details:ContactFull];
}

#pragma mark - New Order

-(void) askNewOrder
{
    if ([self.model.line count] == 0)
    {
        UIAlertController * alert = [UIAlertController
                                     alertControllerWithTitle:@"Pedido vacío"
                                     message:@"El pedido no contiene ningún articulo"
                                     preferredStyle:UIAlertControllerStyleAlert];
        
        UIAlertAction* okAction = [UIAlertAction actionWithTitle:@"Ok"
                                                           style:UIAlertActionStyleDefault
                                                         handler:nil];
        [alert addAction: okAction];
        
        [self presentViewController:alert animated:YES completion:nil];
        
    }
    else
    {
        UIAlertController * alert = [UIAlertController
                                     alertControllerWithTitle:@"Guardar"
                                     message:@"Una vez guardado el pedido no se podrá modificar ¿está seguro?"
                                     preferredStyle:UIAlertControllerStyleAlert];
        
        UIAlertAction* cancelAction = [UIAlertAction actionWithTitle:@"Cancelar"
                                                               style:UIAlertActionStyleDefault
                                                             handler:nil];
        [alert addAction: cancelAction];
        
        UIAlertAction* okAction = [UIAlertAction actionWithTitle:@"Guardar"
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction * action)
                                   {
                                       [self newOrder];
                                   }];
        [alert addAction: okAction];
        
        [self presentViewController:alert animated:YES completion:nil];
    }
}

-(void) newOrder
{
    ModelStack *sharedDataStack = [ModelStack sharedInstance];
    NSManagedObjectContext *context = sharedDataStack.persistentContainer.viewContext;
    
    self.model = [[Order alloc] initWithContext:context];
    [self.model setInitialValues];
    
    [[ModelStack sharedInstance] saveContext];
}

#pragma mark - Gestures

- (void)handleDiscountPan:(UIPanGestureRecognizer *)sender { 
    if([sender state] == UIGestureRecognizerStateBegan)
    {
        self.tempXYPan = [sender translationInView:sender.view];
    }
    if([sender state] != UIGestureRecognizerStateEnded)
    {
        CGPoint point = [sender translationInView:sender.view];
        
        if(point.y > (self.tempXYPan.y +5))
        {
            [self decreaseDiscount];
            self.tempXYPan = point;
        }
        else if(point.y < (self.tempXYPan.y -5))
        {
            [self increaseDiscount];
            self.tempXYPan = point;
        }
    }
}

- (IBAction)increaseDiscount
{
    if(self.model.discount < 100)
    {
        self.model.discount = self.model.discount + 1.0;
    }
}

- (IBAction)decreaseDiscount
{
    if(self.model.discount > 0)
    {
        self.model.discount = self.model.discount - 1.0;
    }
}

- (IBAction)handleHideKeyboard:(UITapGestureRecognizer *)sender
{
    [self.orderCommentsTextView endEditing:YES];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [self.model.line count];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    self.indexPath = indexPath;
    [self performSegueWithIdentifier:@"articleOrderSegue" sender:self];
}

- (OrderLineTableViewCell *)tableView:(UITableView *)tableView
                cellForRowAtIndexPath:(NSIndexPath *)indexPath
{    
    NSSortDescriptor *sort = [[NSSortDescriptor alloc] initWithKey:@"article.articleNumber" ascending:YES];
    NSArray *array = [[self.model.line allObjects] sortedArrayUsingDescriptors:@[sort]];
    
    OrderLine *line = [array objectAtIndex:indexPath.row];
    
    OrderLineTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"articleOrderTableCell"];
    
    [cell setData:line];
    
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 70.0;
}

- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        ModelStack *sharedDataStack = [ModelStack sharedInstance];
        NSManagedObjectContext *context = sharedDataStack.persistentContainer.viewContext;
        
        NSSortDescriptor *sort = [[NSSortDescriptor alloc] initWithKey:@"article.articleNumber" ascending:YES];
        NSArray *array = [[self.model.line allObjects] sortedArrayUsingDescriptors:@[sort]];
        
        OrderLine* deletedLine = [array objectAtIndex:indexPath.row];
        [context deleteObject:deletedLine];
        
        [self.model removeLineObject:deletedLine];
        [sharedDataStack saveContext];
    }
}

- (UITableViewCellEditingStyle)tableView:(UITableView *)aTableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return _editable?UITableViewCellEditingStyleDelete:UITableViewCellEditingStyleNone;
}

#pragma mark - CommentsTextView delgate

-(void)textViewDidBeginEditing:(UITextView *)textView
{
    [UIView animateWithDuration:.5f animations:^{
        [self.view setFrame:CGRectMake(self.view.frame.origin.x,self.view.frame.origin.y-200,self.view.frame.size.width,self.view.frame.size.height)];
    }];
}

-(void)textViewDidEndEditing:(UITextView *)textView
{
    self.model.comments = textView.text;
    [UIView animateWithDuration:.5f animations:^{
        [self.view setFrame:CGRectMake(self.view.frame.origin.x,self.view.frame.origin.y+200,self.view.frame.size.width,self.view.frame.size.height)];
    }];
}

#pragma mark - Customer, peoplepicker delegate

-(void)selectCustomer
{
    //We need permissions
    if( [CNContactStore authorizationStatusForEntityType:CNEntityTypeContacts] == CNAuthorizationStatusNotDetermined)
    {
        CNContactStore * contactStore = [[CNContactStore alloc] init];
        [contactStore requestAccessForEntityType:CNEntityTypeContacts completionHandler:^(BOOL granted, NSError * _Nullable error) {
            if(granted){
                [self showPeoplePicker];
            }
        }];
    }
    else if( [CNContactStore authorizationStatusForEntityType:CNEntityTypeContacts]== CNAuthorizationStatusAuthorized)
    {
        [self showPeoplePicker];
    }
}

-(void) showPeoplePicker
{
    CNContactPickerViewController *contactPicker = [[CNContactPickerViewController alloc] init];
    
    contactPicker.delegate = self;
    contactPicker.displayedPropertyKeys = (NSArray *)CNContactGivenNameKey;
    
    [self presentViewController:contactPicker
                       animated:YES
                     completion:nil];
}

-(void)contactPicker:(CNContactPickerViewController *)picker didSelectContact:(CNContact *)contact
{
    [self dismissViewControllerAnimated:YES completion:nil];
    self.model.customerId = contact.identifier;
}

#pragma mark - PDF

-(void) viewPDF
{
    if ([self.model.line count] == 0)
    {
        UIAlertController * alert = [UIAlertController
                                     alertControllerWithTitle:@"Pedido vacío"
                                     message:@"El pedido no contiene ningún articulo"
                                     preferredStyle:UIAlertControllerStyleAlert];
        
        UIAlertAction* okAction = [UIAlertAction actionWithTitle:@"Ok"
                                                           style:UIAlertActionStyleDefault
                                                         handler:nil];
        [alert addAction: okAction];
        
        [self presentViewController:alert animated:YES completion:nil];
        
    }else
    {
        [self performSegueWithIdentifier:@"pdfSegue" sender:self];
    }
}


#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
    if ([segue.identifier isEqualToString:@"articleOrderSegue"] &&
        self.indexPath)
    {
        NSSortDescriptor *sort = [[NSSortDescriptor alloc] initWithKey:@"article.articleNumber" ascending:YES];
        NSArray *array = [[self.model.line allObjects] sortedArrayUsingDescriptors:@[sort]];
        
        ArticleViewController *destViewController = segue.destinationViewController;
        destViewController.orderLineModel = [array objectAtIndex:self.indexPath.row];
    }
    else if ([segue.identifier isEqualToString:@"pdfSegue"] &&
             self.model)
    {
        PdfViewController *destViewController = segue.destinationViewController;
        destViewController.model = self.model;
    }
}


@end
