//
//  Catalog+CoreDataClass.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 04.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//
//

#import "Catalog+CoreDataClass.h"
#import "ModelStack.h"
#import "ArticleCatalog+CoreDataClass.h"

@implementation Catalog

@synthesize delegate;

+(Catalog*) defaultCatalogModel
{
    static Catalog *catalog = nil;
    
    if(catalog == nil)
    {
        ModelStack *sharedDataStack = [ModelStack sharedInstance];
        NSManagedObjectContext *context = sharedDataStack.persistentContainer.viewContext;
        
        NSFetchRequest *request = [Catalog fetchRequest];
        
        NSArray* catalogs = [context executeFetchRequest:request error:nil];
        
        //First time the app runs a singleton Catalog must be created
        if([catalogs count] == 0)
        {
            catalog = [[Catalog alloc] initWithContext: context];
            [sharedDataStack saveContext];
            
        }else
        {
            catalog = [catalogs objectAtIndex:0];
        }
    }
    
    return catalog;
}

- (void) refreshFromServer
{
    //Download the server's catalog asynchronously informing the main thread about changes
    
    [self.delegate willStartDownloadingCatalog];
    
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        
        NSString *dataURL = [NSString stringWithFormat:@"%@/php/json.php", [[NSUserDefaults standardUserDefaults] objectForKey:@"databaseURL"]];
        NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:dataURL]];
        
        dispatch_group_t group = dispatch_group_create();
        dispatch_group_enter(group);
        
        __block NSData *data = nil;
        __block NSError *error = nil;
        
        NSURLSession *session = [NSURLSession sharedSession];
        [[session dataTaskWithRequest:request
                    completionHandler:^(NSData *_data,
                                        NSURLResponse *_response,
                                        NSError *_error) {
                        
                        data = _data;
                        error = _error;
                        dispatch_group_leave(group);
                    }] resume];
        
        dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
        
        if (data != nil) {
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.delegate didStartDownloadingCatalog];
            });
            
            NSArray * JSONObjects = [NSJSONSerialization JSONObjectWithData:data
                                                                    options:kNilOptions
                                                                      error:&error];
            if (JSONObjects != nil) {
                
                ModelStack *sharedDataStack = [ModelStack sharedInstance];
                NSManagedObjectContext *context = sharedDataStack.persistentContainer.viewContext;
                
                NSManagedObjectContext *temporaryContext = [[NSManagedObjectContext alloc]initWithConcurrencyType:NSPrivateQueueConcurrencyType];
                temporaryContext.parentContext = context;
                
                [temporaryContext performBlock:^{
                    
                    NSFetchRequest *fetchRequest = [Catalog fetchRequest];
                    
                    NSArray* catalogs = [temporaryContext executeFetchRequest:fetchRequest error:nil];
                    Catalog *cataloginTemporaryContext = [catalogs objectAtIndex:0];
                    
                    for (ArticleCatalog *articleToDelete in cataloginTemporaryContext.articleCatalog) {
                        [temporaryContext deleteObject:articleToDelete];
                        [self removeArticleCatalogObject:articleToDelete];
                    }
                    
                    __block NSError *error= nil;
                    NSInteger i = 0;
                    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                                         NSUserDomainMask, YES);
                    NSString *documentsDirectory = [paths objectAtIndex:0];
                    
                    [[NSFileManager defaultManager] createDirectoryAtPath:[documentsDirectory stringByAppendingPathComponent:@"newCatalog" ] withIntermediateDirectories:NO attributes:nil error:nil];
                    
                    
                    for(NSDictionary *dict in JSONObjects)
                    {                
                        ArticleCatalog *article = [[ArticleCatalog alloc] initWithContext: temporaryContext];
                        
                        article.articleNumber = [dict objectForKey:@"articleNumber"];
                        
                        NSString *pictureURL = [NSString stringWithFormat:@"%@/imagenes/%@",[[NSUserDefaults standardUserDefaults] objectForKey:@"databaseURL"],[article pictureFileName]];
                        NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:pictureURL]];
                        
                        __block NSHTTPURLResponse* response = nil;
                        __block NSData *pictureData = nil;
                        
                        dispatch_group_t group = dispatch_group_create();
                        dispatch_group_enter(group);
                        
                        NSURLSession *session = [NSURLSession sharedSession];
                        [[session dataTaskWithRequest:request
                                    completionHandler:^(NSData *_data,
                                                        NSURLResponse *_response,
                                                        NSError *_error) {
                                        pictureData = _data;
                                        response = (NSHTTPURLResponse *) _response;
                                        error = _error;
                                        
                                        dispatch_group_leave(group);
                                    }] resume];
                        
                        dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
                        
                        if ([response statusCode] != 404)
                        {
                            NSString *path = [[documentsDirectory stringByAppendingPathComponent:@"newCatalog"] stringByAppendingPathComponent:[article pictureFileName]];
                            [pictureData writeToFile:path atomically:YES];
                            
                            NSURL *url = [NSURL fileURLWithPath: path];
                            bool success = [url setResourceValue: [NSNumber numberWithBool: YES] forKey: NSURLIsExcludedFromBackupKey error: &error];
                            
                            if (!success)
                            {
                                dispatch_async(dispatch_get_main_queue(), ^{
                                    [self.delegate didErrorDownloadingCatalog:[NSString stringWithFormat:@"Error excluyendo archivo de copia de serguridad.\n\n%@",error.localizedDescription]];
                                });
                            }
                            
                            article.hasPicture = [NSNumber numberWithBool:YES];
                        }
                        
                        NSNumberFormatter * f = [[NSNumberFormatter alloc] init];
                        [f setDecimalSeparator:@"."];
                        [f setGroupingSeparator:@""];
                        
                        article.price=[f numberFromString:[dict objectForKey:@"price"]].floatValue;
                        article.descriptionText = [dict objectForKey:@"descriptionText"];
                        article.catalog = cataloginTemporaryContext;
                        
                        [cataloginTemporaryContext addArticleCatalogObject:article];
                        
                        dispatch_async(dispatch_get_main_queue(), ^{
                            [self.delegate didChangeProgress:[NSNumber numberWithFloat:(float)i/[JSONObjects count]]];
                        });
                        i++;
                    }
                    
                    if (![temporaryContext save:&error]) {
                        dispatch_async(dispatch_get_main_queue(), ^{
                            [self.delegate didErrorDownloadingCatalog:[NSString stringWithFormat:@"No fue posible guardar la base de datos (código mensaje de error:%ld)",(long)error.code]];
                        });
                    }else{
                        [sharedDataStack saveContext];
                        
                        [[NSFileManager defaultManager] removeItemAtPath:[documentsDirectory stringByAppendingPathComponent:@"actualCatalog"] error:&error];
                        [[NSFileManager defaultManager] moveItemAtPath:[documentsDirectory stringByAppendingPathComponent:@"newCatalog"]
                                                                toPath:[documentsDirectory stringByAppendingPathComponent:@"actualCatalog"]
                                                                 error:&error];
                        
                        dispatch_async(dispatch_get_main_queue(), ^{
                            [self.delegate didFinishDownloadingCatalog];
                        });
                    }
                }];
                
            }else{
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self.delegate didErrorDownloadingCatalog:[NSString stringWithFormat:@"La base de datos descargada no es válida. Mensaje de error:\n\n%@",error.localizedDescription]];
                });
            }
        }else{
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.delegate didErrorDownloadingCatalog:[NSString stringWithFormat:@"La base de datos no se ha podido descargar, comprueba que estas conectado a la red, el servidor esta online y la dirección es correcta (%@)", dataURL]];
            });
        }
    });
}

@end


