//
//  Order+CoreDataClass.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 18.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//
//

#import "Order+CoreDataClass.h"
#import "ModelStack.h"
#import "OrderLine+CoreDataClass.h"
#import "ArticleOrder+CoreDataClass.h"

@implementation Order

@synthesize total;
@synthesize  discountAmount;
@synthesize  totalWithoutDiscount;

- (void)addOrderObservers {
    [self addObserver:self
           forKeyPath:@"line"
              options:NSKeyValueObservingOptionNew
              context:nil];
    
    [self addObserver:self
           forKeyPath:@"customerId"
              options:NSKeyValueObservingOptionNew
              context:nil];
    
    [self addObserver:self
           forKeyPath:@"discount"
              options:NSKeyValueObservingOptionNew
              context:nil];
    
    [self addObserver:self
           forKeyPath:@"comments"
              options:NSKeyValueObservingOptionNew
              context:nil];
}

- (void) awakeFromInsert
{
    [super awakeFromInsert];
    
    [self addOrderObservers];
}

-(void) awakeFromFetch
{
    [super awakeFromFetch];
    
    [self addOrderObservers];
    
    [self recalculateOrder];
}

-(void)observeValueForKeyPath:(NSString*)keyPath
                     ofObject:(id)object
                       change:(NSDictionary*)change
                      context:(void*)context
{
    if ([keyPath  isEqual: @"line"] || [keyPath  isEqual: @"discount"]) {
        [self recalculateOrder];
    }
    [[ModelStack sharedInstance] saveContext];
}

+ (Order *)activeOrder
{
    NSNumber *orderId = nil;
    
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    
    orderId = [defaults objectForKey:@"highestOrder"];
    
    if (orderId == nil) {
        // Esto quiere decir que es la primera vez que se llama a la App
        // Ponemos un valor por defecto
        orderId = [NSNumber numberWithInt:1000];
        [defaults setObject:orderId
                     forKey:@"highestOrder"];
        [defaults synchronize];
    }
    
    ModelStack *sharedDataStack = [ModelStack sharedInstance];
    NSManagedObjectContext *context = sharedDataStack.persistentContainer.viewContext;
    
    NSPredicate *resultPredicate = [NSPredicate predicateWithFormat:@"orderId == %@",orderId];

    NSFetchRequest *request = [Order fetchRequest];
    [request setPredicate:resultPredicate];
    
    NSArray* orders = [context executeFetchRequest:request error:nil];
    Order * order;
    
    if ([orders count] == 0) {
        order = [[Order alloc] initWithContext: context];
        [order setInitialValues];
        [sharedDataStack saveContext];
    }else{
        order = [orders objectAtIndex:0];
        [order recalculateOrder];
    }
    
    return order;
}

-(NSNumber *) totalArticles
{
    NSNumber *i = 0;
    
    for (OrderLine* line in self.line) {
        i = [NSNumber numberWithInt: (int)[i integerValue] + line.amount];
    }
    
    return i;
}

-(void) recalculateOrder
{
    self.totalWithoutDiscount = [NSNumber numberWithFloat:0.0];
    for (OrderLine* line in self.line) {
        self.totalWithoutDiscount = [NSNumber numberWithFloat:
                                     self.totalWithoutDiscount.floatValue + [line totalPrice].floatValue];
    }
    
    self.discountAmount = [NSNumber numberWithFloat:self.totalWithoutDiscount.floatValue * self.discount * 0.01];
    
    self.total = [NSNumber numberWithFloat:self.totalWithoutDiscount.floatValue-self.discountAmount.floatValue];
}

-(void) setInitialValues
{
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    int16_t highestOrderID = [[defaults objectForKey:@"highestOrder"] intValue];
    highestOrderID++;
    
    self.orderId = highestOrderID;
    self.date = [NSDate date];
    self.discount = 0;
    self.employeeId =  [defaults objectForKey:@"employeeId"];
    
    [defaults setObject:[NSNumber numberWithInt:highestOrderID ]
                 forKey:@"highestOrder"];
    
    [defaults synchronize];
}

- (void)addArticleToOrder:(nonnull Article *)anArticle
                   amount:(nonnull NSNumber *)anAmount
                 comments:(nonnull NSString *)someComments
                   margin:(nonnull NSNumber *)aMargin
{
    OrderLine *line = [[OrderLine alloc]  initWithContext: self.managedObjectContext];
    ArticleOrder *article = [[ArticleOrder alloc]  initWithContext: self.managedObjectContext];
    
    article.articleNumber = anArticle.articleNumber;
    article.descriptionText = anArticle.descriptionText;
    article.price = anArticle.price;
    article.hasPicture = anArticle.hasPicture;
    article.orderLine = line;
    
    line.order = self;
    line.article = article;
    line.amount = (int16_t)anAmount.intValue;
    line.comments = someComments;
    line.margin = aMargin.integerValue;
    line.order = self;
    
    [self addLineObject:line];
    
    [[ModelStack sharedInstance] saveContext];
}

- (nonnull NSString *)guiOrderId { 
    return [NSString stringWithFormat:@"%@-%d", self.employeeId, self.orderId];
}

@end
