//
//  HorusUtils.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 07.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//

#import "HorusUtils.h"

@import Contacts;

@implementation HorusUtils

+(NSString *) priceString:(NSNumber *)aNumber
                    units:(BOOL)units
{
    NSNumberFormatter* numberFormatter = [[NSNumberFormatter alloc] init];
    [numberFormatter setFormatterBehavior: NSNumberFormatterBehavior10_4];
    [numberFormatter setNumberStyle: NSNumberFormatterDecimalStyle];
    [numberFormatter setMinimumFractionDigits:2];
    [numberFormatter setMaximumFractionDigits:2];
    
    NSMutableString* string = [NSMutableString stringWithFormat:@"%@",[numberFormatter stringFromNumber: [NSNumber numberWithFloat:aNumber.floatValue]]];
    
    if(units)
    {
        [string appendString:@" €"];
    }
    
    return string;
}

+(NSString*)contactStringWith:(NSString*)contactId
                      details:(ContactDetails)details
{
    CNEntityType entityType = CNEntityTypeContacts;
    
    if( [CNContactStore authorizationStatusForEntityType:entityType] == CNAuthorizationStatusDenied)
    {
        return @"Permita el acceso a contactos en \"Ajustes\"";
    }
    else if(contactId == nil || [contactId length] == 0)
    {
        return @"";
    }
    
    NSMutableString *string = [NSMutableString stringWithFormat:@"Contacto no encontrado (%@)",contactId];
    
    //Create repository objects contacts
    CNContactStore *contactStore = [[CNContactStore alloc] init];
    
    NSArray *keys = [[NSArray alloc]initWithObjects:CNContactIdentifierKey,
                     CNContactGivenNameKey,
                     CNContactFamilyNameKey,
                     CNContactOrganizationNameKey,
                     CNContactEmailAddressesKey,
                     CNContactPostalAddressesKey,
                     CNContactPhoneNumbersKey, nil];
    
    // Create a request object
    CNContactFetchRequest *request = [[CNContactFetchRequest alloc] initWithKeysToFetch:keys];
    
    [contactStore enumerateContactsWithFetchRequest:request
                                              error:nil
                                         usingBlock:^(CNContact* __nonnull contact, BOOL* __nonnull stop)
     {
         if([contact.identifier isEqualToString:contactId])
         {
             [string setString:@""];
             
             switch (details) {
                 case ContactNameCompany:
                 case ContactFull:
                 {
                     [string appendString:contact.givenName];
                     
                     if(![string isEqualToString:@""])
                     {
                         [string appendString:@" "];
                     }
                     
                     [string appendString:contact.familyName];
                     
                     if([string isEqualToString:@""])
                     {
                         [string appendString:contact.organizationName];
                         
                     }else if([contact.organizationName length] > 0){
                         [string appendString:@" - "];
                         [string appendString:contact.organizationName];
                     }
                     
                     
                     if(details == ContactFull)
                     {
                         if(![string isEqualToString:@""])
                         {
                             [string appendString:@"\n"];
                         }
                         
                         if( contact.postalAddresses.count > 0 )
                         {
                             CNPostalAddress *address = [[contact.postalAddresses firstObject] value];
                             [string appendString:[NSString stringWithFormat:@"%@\n%@ - %@, %@\n", address.street, address.postalCode, address. city, address.state]];
                         }
                         
                     }else
                     {
                         break;
                     }
                 }
                 case ContactEmail:
                     if( contact.emailAddresses.count > 0 )
                     {
                         [string appendString:[[contact.emailAddresses firstObject] value]];
                     }
                     break;
                 default:
                     break;
             }
         }
     }];
    
    return string;
}

@end
