//
//  PdfRenderer.m
//  horussalesman
//
//  Created by Miguel Cholbi Collados on 20.11.17.
//  Copyright © 2017 Horus Joyeros. All rights reserved.
//

#import "PdfRenderer.h"
#import "OrderLine+CoreDataClass.h"
#import "ArticleOrder+CoreDataClass.h"
#import "HorusUtils.h"
#import <CoreText/CoreText.h>

#define MAX_LINES_PER_PAGE 5

@implementation PdfRenderer

#pragma - mark Public getters

+(NSData*) drawOrderPDFtoDataWith:(Order*)aModel
{
    NSMutableData* pdfData = [NSMutableData data];
    UIGraphicsBeginPDFContextToData(pdfData, CGRectZero, nil);
    [self drawOrderPDF:aModel];
    
    return pdfData;
}

+(void) drawOrderPDFtoFileWith:(Order*)aModel
{
    UIGraphicsBeginPDFContextToFile([self getPDFFileName], CGRectZero, nil);
    [self drawOrderPDF:aModel];
}

+(NSString*)getPDFPath
{
    NSArray *arrayPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory,
                                                              NSUserDomainMask,
                                                              YES);
    return [arrayPaths objectAtIndex:0];
}

+(NSString*)getPDFFileName
{
    NSString* pdfFileName = [[self getPDFPath] stringByAppendingPathComponent:@"order.pdf"];
    
    return pdfFileName;
}

#pragma mark - Order's elements drawers

+(void) drawOrderPDF:(Order*)aModel
{
    NSArray* objects = [[NSBundle mainBundle] loadNibNamed:@"Page" owner:nil options:nil];
    
    UIView *page = [objects objectAtIndex:0];
    
    [self drawHeader:page model:aModel];
    [self drawLines:page model:aModel];
    UIGraphicsEndPDFContext();
}

+(void) drawHeader:(UIView*)view
             model:(Order*)aModel
{
    UIGraphicsBeginPDFPageWithInfo(CGRectMake(0, 0, 612, 792), nil);
    UIImage *backgroundTemplate = [UIImage imageNamed:@"orderTemplate.jpg"];
    [backgroundTemplate drawInRect:CGRectMake(0, 0, 612, 792)];
    
    UILabel *orderIDLabel = (UILabel *)[view viewWithTag:10];
    UILabel *dateLabel = (UILabel *)[view viewWithTag:20];
    UITextView *customerTextView = (UITextView *)[view viewWithTag:30];
    
    [self drawText:[NSString stringWithFormat:@"%@",[aModel guiOrderId]] inFrame:orderIDLabel.frame bold:YES];
    [self drawText:[NSDateFormatter localizedStringFromDate:aModel.date
                                                  dateStyle:NSDateFormatterMediumStyle
                                                  timeStyle:NSDateFormatterNoStyle] inFrame:dateLabel.frame bold:NO];
    [self drawText:[HorusUtils contactStringWith:aModel.customerId
                                         details:ContactFull] inFrame:customerTextView.frame bold:NO];
}

+(void) drawLines:(UIView*)aView
            model:(Order*)aModel
{
    NSSortDescriptor *sort = [[NSSortDescriptor alloc] initWithKey:@"article.articleNumber" ascending:YES];
    NSArray *array = [[aModel.line allObjects] sortedArrayUsingDescriptors:@[sort]];
    
    UIView *lineView = (UIView *)[aView viewWithTag:100];
    CGRect originFrame = lineView.frame;
    
    for(NSInteger index=0,page=0,row=0;index < [array count];index++,row++)
    {
        OrderLine* line = [array objectAtIndex:index];
        
        if (row == MAX_LINES_PER_PAGE)
        {
            UILabel *moreLabel = (UILabel *)[aView viewWithTag:190];
            [self drawText:moreLabel.text inFrame:moreLabel.frame bold:NO];
            
            [self drawHeader:aView model:aModel];
            row = 0;
            page++;
            lineView.frame = originFrame;
        }
        
        UIImageView *image = (UIImageView *)[aView viewWithTag:110];
        UILabel *amountLabel = (UILabel *)[aView viewWithTag:120];
        UILabel *articleNumberLabel = (UILabel *)[aView viewWithTag:130];
        UILabel *descriptionLabel = (UILabel *)[aView viewWithTag:140];
        UILabel *commentsLabel = (UILabel *)[aView viewWithTag:150];
        UILabel *pricePerUnitLabel = (UILabel *)[aView viewWithTag:160];
        UILabel *priceTotalLabel = (UILabel *)[aView viewWithTag:170];
        
        [[[line article] picture]drawInRect:[self traslateFrame:image.frame to:lineView.frame]];
        [self drawText:[NSString stringWithFormat:@"%@",line.article.articleNumber] inFrame:[self traslateFrame:articleNumberLabel.frame to:lineView.frame] bold:YES];
        
        [self drawText:[NSString stringWithFormat:@"%d",line.amount] inFrame:[self traslateFrame:amountLabel.frame to:lineView.frame] bold:NO];
        [self drawText:[NSString stringWithFormat:@"%@",line.article.descriptionText] inFrame:[self traslateFrame:descriptionLabel.frame to:lineView.frame] bold:NO];
        [self drawText:[NSString stringWithFormat:@"%@",line.comments==nil?@"":line.comments] inFrame:[self traslateFrame:commentsLabel.frame to:lineView.frame] bold:NO];
        [self drawText:[HorusUtils priceString:line.unitPrice units:YES] inFrame:[self traslateFrame:pricePerUnitLabel.frame to:lineView.frame] bold:NO];
        [self drawText:[HorusUtils priceString:line.totalPrice units:YES] inFrame:[self traslateFrame:priceTotalLabel.frame to:lineView.frame] bold:NO];
        
        lineView.frame = CGRectMake(lineView.frame.origin.x, lineView.frame.origin.y  + lineView.frame.size.height + 5, lineView.frame.size.width, lineView.frame.size.height);
    }
    
    [self drawFooter:aView model:aModel];
    
}

+(void) drawFooter:(UIView*)view
             model:(Order*)aModel
{
    UILabel *piezasLabel = (UILabel *)[view viewWithTag:210];
    UILabel *totalWithoutdiscount = (UILabel *)[view viewWithTag:220];
    UILabel *discountLabel = (UILabel *)[view viewWithTag:230];
    UILabel *discountTotalLabel = (UILabel *)[view viewWithTag:240];
    UILabel *totalLabel = (UILabel *)[view viewWithTag:250];
    UITextView *commentsTextView = (UITextView *)[view viewWithTag:260];
    
    [self drawText:[NSString stringWithFormat:@"%d",(int)[[aModel totalArticles]integerValue]] inFrame:piezasLabel.frame bold:NO];
    
    if (aModel.discount > 0) {
        [self drawText:[HorusUtils priceString: aModel.totalWithoutDiscount units:YES] inFrame:totalWithoutdiscount.frame bold:NO];
        [self drawText:[HorusUtils priceString: aModel.discountAmount units:YES] inFrame:discountTotalLabel.frame bold:NO];
        [self drawText:[NSString stringWithFormat:@"%d", aModel.discount] inFrame:discountLabel.frame bold:NO];
    }
    
    [self drawText:[HorusUtils priceString: aModel.total units:YES] inFrame:totalLabel.frame bold:YES];
    [self drawText:aModel.comments==nil?@"":aModel.comments inFrame:commentsTextView.frame bold:NO];
    
}

#pragma mark - Auxiliar drawing methods

+(void)drawText:(NSString*)textToDraw
        inFrame:(CGRect)frameRect
           bold:(BOOL)wantBold
{
    
    CFStringRef stringRef = (__bridge CFStringRef)textToDraw;
    
    // Prepare the text using a Core Text Framesetter
    CFMutableAttributedStringRef attString =  CFAttributedStringCreateMutable(kCFAllocatorDefault, CFStringGetLength(stringRef));
    CFAttributedStringReplaceString(attString, CFRangeMake(0, 0), (CFStringRef)stringRef);
    CTFontRef font = CTFontCreateWithName((CFStringRef)@"CourierNewPSMT", 13.0f, nil);
    CTFontRef newFont = CTFontCreateCopyWithSymbolicTraits(font, 0.0, NULL, (wantBold?kCTFontBoldTrait:0), kCTFontBoldTrait);
    
    CFAttributedStringSetAttribute(attString, CFRangeMake(0, CFAttributedStringGetLength(attString)), kCTFontAttributeName, newFont);
    CTFramesetterRef framesetter = CTFramesetterCreateWithAttributedString(attString);
    
    CGMutablePathRef framePath = CGPathCreateMutable();
    CGPathAddRect(framePath, NULL, frameRect);
    
    // Get the frame that will do the rendering.
    CFRange currentRange = CFRangeMake(0, 0);
    CTFrameRef frameRef = CTFramesetterCreateFrame(framesetter, currentRange, framePath, NULL);
    CGPathRelease(framePath);
    
    // Get the graphics context.
    CGContextRef currentContext = UIGraphicsGetCurrentContext();
    
    // Put the text matrix into a known state. This ensures
    // that no old scaling factors are left in place.
    CGContextSetTextMatrix(currentContext, CGAffineTransformIdentity);
    
    // Core Text draws from the bottom-left corner up, so flip
    // the current transform prior to drawing.
    CGContextTranslateCTM(currentContext, 0, frameRect.size.height+frameRect.origin.y*2);
    CGContextScaleCTM(currentContext, 1.0, -1.0);
    
    // Draw the frame.
    CTFrameDraw(frameRef, currentContext);
    
    CGContextScaleCTM(currentContext, 1.0, -1.0);
    CGContextTranslateCTM(currentContext, 0, (-1)*(frameRect.size.height+frameRect.origin.y*2));
    
    CFRelease(frameRef);
    CFRelease(framesetter);
}

+(CGRect)traslateFrame:(CGRect)originalFrame to:(CGRect)traslation
{
    return CGRectMake(traslation.origin.x + originalFrame.origin.x,traslation.origin.y + originalFrame.origin.y, originalFrame.size.width, originalFrame.size.height);
}


@end
